﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Globalization;
using System.Linq;

namespace CoffeeStoreAddin.CmdLine
{
    internal class CmdLineParser
    {
        public CmdLineParser(List<ArgItem> supportedArgs)
        {
            SupportedArgs = supportedArgs;
        }

        internal List<ArgItem> SupportedArgs { get; }

        internal Dictionary<string, dynamic> GetTypedArgs(string cmdLineStr)
        {
            Dictionary<string, dynamic> typedArgs = null;
            var rawArgs = GetRawArgs(cmdLineStr);
            if (rawArgs != null)
            {
                typedArgs = new Dictionary<string, dynamic>();
                foreach (var rawArg in rawArgs)
                {
                    var supportedArg = SupportedArgs.FirstOrDefault(a => a.Text == rawArg.Key);
                    if (supportedArg != null)
                    {
                        try
                        {
                            dynamic argVal = Convert.ChangeType(rawArg.Value, supportedArg.ArgType, CultureInfo.InvariantCulture);
                            typedArgs.Add(supportedArg.Text, argVal);
                        }
                        catch (FormatException ex)
                        {
                            Debug.WriteLine($"Unable to convert argument value - '{supportedArg.Text}={rawArg.Value}' to {supportedArg.ArgType.ToString()} ({ex.Message})");
                        }
                    }
                    else
                    {
                        if (rawArg.Key.Length > 0)
                        {
                            Debug.WriteLine($"Unsupported argument name - '{rawArg.Key}={rawArg.Value}'");
                        }
                    }
                }
            }
            return typedArgs;
        }

        private static Dictionary<string, string> GetRawArgs(string cmdLineStr)
        {
            var q = cmdLineStr.Split('/')
                    .Select(arg => arg.Trim().Split('='))
                    .ToDictionary(parts => parts[0].Trim(), parts => parts.ElementAtOrDefault(1)?.Trim());
            return q;
        }
    }
}
