﻿using System;
using Visio = Microsoft.Office.Interop.Visio;

namespace CoffeeStoreAddin
{
    [System.Runtime.InteropServices.ComVisible(true)]
    public class DocEvtSink : Visio.IVisEventProc
    {

        // ReSharper disable once InconsistentNaming
        protected const short visEvtAdd = -32768;

        private readonly DocWrapper _docWrapper;

        public DocEvtSink(DocWrapper docWrapper)
        {
            _docWrapper = docWrapper;
        }


        /// <summary>This method is called by Visio when an event in the
        /// EventList collection has been triggered. This method is an
        /// implementation of IVisEventProc.VisEventProc method.</summary>
        /// <param name="eventCode">Event code of the event that fired</param>
        /// <param name="source">Reference to source of the event</param>
        /// <param name="eventId">Unique identifier of the event object that 
        /// raised the event</param>
        /// <param name="eventSequenceNumber">Relative position of the event in 
        /// the event list</param>
        /// <param name="subject">Reference to the subject of the event</param>
        /// <param name="moreInformation">Additional information for the event
        /// </param>
        /// <returns>False to allow a QueryCancel operation or True to cancel 
        /// a QueryCancel operation. The return value is ignored by Visio unless 
        /// the event is a QueryCancel event.</returns>
        /// <seealso cref="Microsoft.Office.Interop.Visio.IVisEventProc"></seealso>
        public object VisEventProc(short eventCode,
                                   object source,
                                   int eventId,
                                   int eventSequenceNumber,
                                   object subject,
                                   object moreInformation)
        {
            object returnValue = true;

            if (source == null)
                return null;
            
            try
            {
                switch (eventCode)
                {
                    //BeforeSelectionDelete
                    case (short)Visio.VisEventCodes.visEvtCodeBefSelDel:
                        var selSubject = subject as Visio.Selection;
                        if (selSubject != null)
                        {
                            _docWrapper.FurnitureMgr.FlagTablesForArrange(selSubject);
                        }
                        break;

                    //NoEventsPending
                    case (short)Visio.VisEventCodes.visEvtApp + (short)Visio.VisEventCodes.visEvtNonePending:
                        _docWrapper.FurnitureMgr.ProcessFlaggedTables();
                        break;
                }

                

            }
            catch (Exception err)
            {
                System.Diagnostics.Debug.WriteLine(err.Message);
                throw;
            }

            return returnValue;
        }
    }
}
