﻿using System.Collections.Generic;
using Visio = Microsoft.Office.Interop.Visio;

namespace CoffeeStoreAddin
{
    public class DocWrapper
    {
        private DocEvtSink _docEvtSink;
        private List<Visio.Event> _watchedEvents;

        public DocWrapper(int docId, FurnitureManager furnitureMgr)
        {
            Id = docId;
            FurnitureMgr = furnitureMgr;
            VisioEvents_Connect();
        }

        ~DocWrapper()
        {   
            VisioEvents_Disconnect();
        }


        internal int Id { get; }


        internal FurnitureManager FurnitureMgr { get; set; }


        internal void ProcessCmdLine(string cmdStr, bool skipSolutionCheck)
        {
            if (cmdStr != null && (skipSolutionCheck || cmdStr.Contains(ThisAddIn.SolutionIdentity.SolnAndIDArg)))
            {
                var cmdLinePaser = Globals.ThisAddIn.CmdLineParser;
                var typedArgs = cmdLinePaser.GetTypedArgs(cmdStr);
                if (typedArgs.ContainsKey(ThisAddIn.CmdArgs.Cmd))
                {
                    var cmdValue = typedArgs[ThisAddIn.CmdArgs.Cmd] as string;
                    if (!string.IsNullOrWhiteSpace(cmdValue))
                    {
                        switch (cmdValue.ToLower())
                        {
                            case "addchair":
                                FurnitureMgr.AddChair(Globals.ThisAddIn.Application.ActiveWindow.Selection.PrimaryItem);
                                break;
                            case "selectallchairs":
                                FurnitureMgr.SelectAllChairs(Globals.ThisAddIn.Application.ActiveWindow.Selection.PrimaryItem);
                                break;
                        }
                    }
                }  
            }
        }

        private void VisioEvents_Connect()
        {
            var vDoc = GetVisDocInstance();
            var docEventlist = vDoc?.EventList;
            var appEventlist = vDoc?.Application?.EventList;
            if (docEventlist != null)
            {
                _docEvtSink = new DocEvtSink(this);
                if (_watchedEvents == null)
                    _watchedEvents = new List<Visio.Event>();

                //BeforeSelectionDelete
                var evtBefDelSel = docEventlist.AddAdvise((short)Visio.VisEventCodes.visEvtCodeBefSelDel, 
                                                          _docEvtSink, "", "");
                _watchedEvents.Add(evtBefDelSel);

                //NoEventsPending
                var evtNonePending = appEventlist?.AddAdvise((short)Visio.VisEventCodes.visEvtApp 
                                                           + (short)Visio.VisEventCodes.visEvtNonePending, 
                                                           _docEvtSink, "", "");
                _watchedEvents.Add(evtNonePending);
            }
        }

        private void VisioEvents_Disconnect()
        {
            foreach (Visio.Event evt in _watchedEvents)
            {
                evt.Delete();
            }
            _watchedEvents = null;
        }


        private Visio.Document GetVisDocInstance()
        {
            var docs = Globals.ThisAddIn.Application.Documents;
            return docs.Count >= Id ? docs.ItemFromID[Id] : null;
        }  

    }
}
