﻿using System.Collections.Generic;
using System.Linq;
using CoffeeStoreAddin.CmdLine;
using Visio = Microsoft.Office.Interop.Visio;

namespace CoffeeStoreAddin
{
    public partial class ThisAddIn
    {
        internal static class SolutionIdentity
        {
            internal const string CellName = "User.SolutionId";
            internal const string Id = "{FB3B8B03-455F-4AB3-9A3C-A28E19D14A19}";
            internal const string SolnAndIDArg = "/soln=" + Id;  
        }

        internal static class CmdArgs
        {
            internal const string Soln = "soln";
            internal const string Doc = "doc";
            internal const string Page = "page";
            internal const string Shape = "shape";
            internal const string ShapeU = "shapeu";
            internal const string Cmd = "cmd";
        }


        private void ThisAddIn_Startup(object sender, System.EventArgs e)
        {
            CmdLineParser = new CmdLineParser(new List<ArgItem>
            {
                new ArgItem() { Text=CmdArgs.Soln, ArgType=typeof(string) },
                new ArgItem() { Text=CmdArgs.Doc, ArgType=typeof(int) },
                new ArgItem() { Text=CmdArgs.Page, ArgType=typeof(int) },
                new ArgItem() { Text=CmdArgs.Shape, ArgType=typeof(string) },
                new ArgItem() { Text=CmdArgs.ShapeU, ArgType=typeof(string) },
                new ArgItem() { Text=CmdArgs.Cmd, ArgType=typeof(string) }
		    });

            VisioEvents_Connect();
        }

        private void ThisAddIn_Shutdown(object sender, System.EventArgs e)
        {
            VisioEvents_Disconnect();
        }


        internal CmdLineParser CmdLineParser { get; private set; }

        internal List<DocWrapper> Docs { get; set; }

        private void VisioEvents_Connect()
        {
            Application.DocumentCreated += new Visio.EApplication_DocumentCreatedEventHandler(Application_DocumentOpenedOrCreated);
            Application.DocumentOpened += Application_DocumentOpenedOrCreated;
            Application.BeforeDocumentClose += Application_BeforeDocumentClose;
        }

        private void VisioEvents_Disconnect()
        {
            Application.DocumentCreated -= Application_DocumentOpenedOrCreated;
            Application.DocumentOpened -= Application_DocumentOpenedOrCreated;
            Application.BeforeDocumentClose -= Application_BeforeDocumentClose;
        }

        private void Application_DocumentOpenedOrCreated(Visio.Document doc)
        {
            if (IsSolutionDoc(doc))
            {
                if (Docs == null)
                    Docs = new List<DocWrapper>();

                if (Docs.Count == 0)
                    Application.MarkerEvent += Application_MarkerEvent;

                Docs.Add(new DocWrapper(doc.ID, new FurnitureManager()));
            }
        }

        private void Application_MarkerEvent(Visio.Application app, int sequenceNum, string contextString)
        {
            if (contextString != null && contextString.Contains(SolutionIdentity.SolnAndIDArg))
            {
                var targetDoc = Docs?.FirstOrDefault(d => d.Id == Application.ActiveDocument.ID);
                targetDoc?.ProcessCmdLine(contextString, skipSolutionCheck: true);
            }
        }

        private void Application_BeforeDocumentClose(Visio.Document doc)
        {
            if (IsSolutionDoc(doc))
            {
                Docs?.RemoveAll(d => d.Id == doc.ID);
                if (Docs?.Count == 0)
                    Application.MarkerEvent -= Application_MarkerEvent;
            }
        }

        private static bool IsSolutionDoc(Visio.Document vDoc)
        {
            return vDoc?.DocumentSheet.CellExistsU[SolutionIdentity.CellName, (short) Visio.VisExistsFlags.visExistsAnywhere] != 0
                   && vDoc?.DocumentSheet.CellsU[SolutionIdentity.CellName].ResultStrU[Visio.VisUnitCodes.visUnitsString] == SolutionIdentity.Id;
        }


        

        #region VSTO generated code

        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InternalStartup()
        {
            this.Startup += new System.EventHandler(ThisAddIn_Startup);
            this.Shutdown += new System.EventHandler(ThisAddIn_Shutdown);
        }
        
        #endregion
    }
}
