﻿using System.Collections.Generic;
using System.Linq;
using Visio = Microsoft.Office.Interop.Visio;

namespace CoffeeStoreAddin.VisExtensions
{
    static class VisExts
    {
        public static List<Visio.Window> GetDrawingWindows(this Visio.Shape shp)
        { 
            var targetDocName = shp.Document.Name;
            return (from wdw in shp.Application.Windows.Cast<Visio.Window>()
                    where wdw.Type == (short) Visio.VisWinTypes.visDrawing
                    where wdw.Document.Name == targetDocName
                    select wdw).ToList();
        }

        public static Visio.Document DockedStencil(this Visio.Window wdw, string fileName)
        {
            if (string.IsNullOrWhiteSpace(fileName))
                return null;

            System.Array nameArray;
            wdw.DockedStencils(out nameArray);
            for (int index = nameArray.GetLowerBound(0);
                            index <= nameArray.GetUpperBound(0); index++)
            {
                var fullName = nameArray.GetValue(index).ToString();
                var shortName = System.IO.Path.GetFileName(fullName);

                if (shortName == fileName || fullName == fileName)
                {
                    return wdw.Application.Documents[fullName];
                }
            }
            return null;
        }


        internal static bool IsMasterInstanceOf(this Visio.Shape shpIn, string[] mstNameUArr)
        {
            return mstNameUArr.Any(n => n == shpIn?.Master?.NameU);
        }

        internal static bool IsMasterInstanceOf(this Visio.Shape shpIn, string mstNameU)
        {
            return shpIn?.Master?.NameU == mstNameU;
        }

    }
}
